import binascii
import json, datetime
from django.shortcuts import HttpResponse
from django.middleware.csrf import get_token
from django.views.decorators.http import require_GET, require_POST
from apps.auth import models as auth_models
from django.contrib.auth.hashers import make_password, check_password
from apps.api.common import CaesarCypherClass, Base64CypherClass
from apps.api.config import Config

# Create your views here.
config = Config()
base64 = Base64CypherClass()
caesar = CaesarCypherClass()


@require_GET
def gettoken(request):
    """
    获取token
    :param request:
    :return:
    """
    token = get_token(request)
    return HttpResponse(json.dumps({'token': token}), content_type="application/json,charset=utf-8")


@require_POST
def search_user(request):
    """
    查询用户名
    :param request:
    :return:
    """
    if config.getconfig("isCypher"):
        if config.getconfig("CypherMethod") == "caesar":
            username = caesar.caesar_encode(request.POST.get("username"))
            user = auth_models.User.objects.using("cypher").filter(username=username)
        else:
            username = base64.base64_encode_str(base64, request.POST.get("username")).decode('utf-8')
            user = auth_models.User.objects.using("cypher").filter(username=username)
    else:
        username = request.POST.get("username")
        user = auth_models.User.objects.using("default").filter(username=username)
    if user.exists():
        return HttpResponse(True)
    else:
        return HttpResponse(False)


@require_POST
def add_user(request):
    """
    用户注册
    :param request: POST提交注册信息
    :return: 注册结果
    """
    try:
        create_time = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        last_login_time = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        avatar = request.FILES.get("avatar")
        pwd_base64 = base64.base64_decode_str(base64, request.POST.get("pwd"))
        pwd = make_password(pwd_base64)
        if config.getconfig("isCypher"):
            if config.getconfig("CypherMethod") == "caesar":
                username = caesar.caesar_encode(request.POST.get("username"))
                email = caesar.caesar_encode(request.POST.get("email"))
                phone = caesar.caesar_encode(request.POST.get("phone"))
            else:
                username = base64.base64_encode_str(base64, request.POST.get("username")).decode('utf-8')
                email = base64.base64_encode_str(base64, request.POST.get("email")).decode('utf-8')
                phone = base64.base64_encode_str(base64, request.POST.get("phone")).decode('utf-8')
            auth_models.User.objects.using("cypher").create(
                username=username,
                pwd=pwd,
                email=email,
                phone=phone,
                create_time=create_time,
                last_login_time=last_login_time,
                avatar=avatar
            )
        else:
            username = request.POST.get("username")
            email = request.POST.get("email")
            phone = request.POST.get("phone")
            auth_models.User.objects.using("default").create(
                username=username,
                pwd=pwd,
                email=email,
                phone=phone,
                create_time=create_time,
                last_login_time=last_login_time,
                avatar=avatar
            )
        return HttpResponse("添加用户成功")
    except Exception as e:
        return HttpResponse(f"报错了：{e}")


@require_POST
def login_user(request):
    """
    用户登录，验证用户密码是否正确，正确返回菜单，错误返回用户名或密码不正确
    :param request:
    :return:
    """
    try:
        if config.getconfig("isCypher"):
            if config.getconfig("CypherMethod") == "caesar":
                username = caesar.caesar_encode(request.POST.get("username"))
                pwd_input = caesar.caesar_encode(request.POST.get("pwd"))
                pwd_made = auth_models.User.objects.using("cypher").filter(username=username).first()
            else:
                username = base64.base64_encode_str(base64, request.POST.get("username")).decode('utf-8')
                pwd_input = base64.base64_decode_str(base64, request.POST.get("pwd"))
                pwd_made = auth_models.User.objects.using("cypher").filter(username=username).first()
        else:
            username = request.POST.get("username")
            pwd_input = base64.base64_decode_str(base64, request.POST.get("pwd"))
            pwd_made = auth_models.User.objects.using("default").filter(username=username).first()
        if pwd_made is not None:
            pwd = check_password(pwd_input, pwd_made.pwd)
            if pwd:
                return HttpResponse(True)
            else:
                return HttpResponse("用户名或密码不正确")
        else:
            return HttpResponse("用户名或密码不正确")
    except binascii.Error as e:
        return HttpResponse("base64解码失败")
    except Exception as e:
        return HttpResponse(f"报错了:{e}")
